//		Copyright © 1997	Jens Kilian
//
//		This program is free software; you can redistribute it and/or modify
//		it under the terms of the GNU General Public License as published by
//		the Free Software Foundation; either version 1, or (at your option)
//		any later version.
//
//		This program is distributed in the hope that it will be useful,
//		but WITHOUT ANY WARRANTY; without even the implied warranty of
//		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
//		GNU General Public License for more details.
//
//		You should have received a copy of the GNU General Public License
//		along with this program; if not, write to the Free Software
//		Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

//		Draw and cache the knot's sections.

#ifndef KNOTSECTIONCACHE_H
#define KNOTSECTIONCACHE_H

#include <Bitmap.h>
#include <Point.h>
#include <Rect.h>
#include <View.h>

#include "KnotSection.h"

class KnotSectionCache : private BView {
	typedef BView inherited;
public:

	// Constructor, destructor.
	KnotSectionCache(float edgeLength, float outlineWidth, float fillWidth);
	virtual ~KnotSectionCache(void);

	// Return size of a section.
	double EdgeLength(void) const;
	
	// Return a bitmap for a given section.
	const BBitmap *GetKnotSection(const KnotSection &section);

private:
	KnotSectionCache(const KnotSectionCache &orig);
	KnotSectionCache &operator =(const KnotSectionCache &orig);

	// Drawing methods.
	void DrawKnotSection(const KnotSection &section);

	void DiagonalSection(const KnotSection &section);
	void DiagonalStroke(direction from, direction to, bool mirror);

	void HorizontalSection(const KnotSection &section);
	void HorizontalStroke(direction from, direction to, bool mirror);

	void VerticalSection(const KnotSection &section);
	void VerticalStroke(direction from, direction to, bool mirror);

	void StrokeCorners(const KnotSection &section);
	void StrokeLeftTop(void);
	void StrokeRightBottom(void);

	void StrokeDDD(int orientation);
	void StrokeDHD(int orientation);
	void StrokeDHH(int orientation);
	void StrokeDHV(int orientation);
	
	void StrokeHDD(int orientation);
	void StrokeHDH(int orientation);
	void StrokeHDV(int orientation);
	void StrokeHHH(int orientation);
	void StrokeHHV(int orientation);
	void StrokeHVV(int orientation);

	// Data members.
	const float mSize;
	const BPoint mCenter;
	const float mOutlineWidth;
	const float mFillWidth;
	
	BBitmap mCanvas;
	BBitmap *mpCache[3][4][4][4][4];
};

inline double
KnotSectionCache::EdgeLength(void) const
{
	return mSize * 2.0;
}

#endif
