//		Copyright © 1997	Jens Kilian
//
//		This program is free software; you can redistribute it and/or modify
//		it under the terms of the GNU General Public License as published by
//		the Free Software Foundation; either version 1, or (at your option)
//		any later version.
//
//		This program is distributed in the hope that it will be useful,
//		but WITHOUT ANY WARRANTY; without even the implied warranty of
//		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
//		GNU General Public License for more details.
//
//		You should have received a copy of the GNU General Public License
//		along with this program; if not, write to the Free Software
//		Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

//		Select a color of random hue, within a given range of lightness
//		and saturation.

#include <cstdlib>
#include <GraphicsDefs.h>

#include "RandomColor.h"

inline int
rgb(int q1, int q2, int hue)
{
	hue = (hue + 360) % 360;
	if (hue < 60) {
		return q1 + (q2-q1) * hue / 60;

	} else if (hue < 180) {
		return q2;

	} else if (hue < 240) {	
		return q1 + (q2-q1) * (240 - hue) / 60;
	
	} else {
		return q1;
	}
}

RandomColor::RandomColor(float minL, float maxL, float minS, float maxS)
{
	int hue = rand() % 360;
	int lightness = (int)(255.0*minL) + rand() % (int)(256.0*(maxL-minL));
	int saturation = (int)(255.0*minS) + rand() % (int)(256.0*(maxS-minS));

	// HSL to RGB conversion.  Sheer magic, don't ask me :-)
	int p2 = (lightness <= 128)
		? lightness * (saturation + 256)
		: lightness + saturation - lightness*saturation;
	int p1 = 2*lightness - p2;
	
	if (saturation == 0) {
		mColor.red = lightness;
		mColor.green = lightness;
		mColor.blue = lightness;
	} else {
		mColor.red = rgb(p1, p2, hue + 120);
		mColor.green = rgb(p1, p2, hue);
		mColor.blue = rgb(p1, p2, hue - 120);
	}
}
